@@ -8,7 +8,7 @@ Pelix is a Python framework that aims to act as OSGi as much as possible
 :author: Thomas Calmant
 :copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.5.7
+:version: 0.5.8
 :status: Beta
 
 ..
@@ -29,7 +29,7 @@ Pelix is a Python framework that aims to act as OSGi as much as possible
 """
 
 # Module version
-__version_info__ = (0, 5, 7)
+__version_info__ = (0, 5, 8)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -638,7 +638,8 @@ class Framework(Bundle):
         :return: the list of all installed bundles
         """
         with self.__bundles_lock:
-            return list(self.__bundles.values())
+            return list(self.__bundles[bundle_id]
+                        for bundle_id in sorted(self.__bundles.keys()))
 
     def get_properties(self):
         """
@@ -1468,13 +1469,16 @@ class FrameworkFactory(object):
             return cls.__singleton == framework
 
     @classmethod
-    def delete_framework(cls, framework):
+    def delete_framework(cls, framework=None):
         """
         Removes the framework singleton
 
         :return: True on success, else False
         """
-        if cls.__singleton is framework:
+        if framework is None:
+            framework = cls.__singleton
+
+        if framework is cls.__singleton:
             # Stop the framework
             try:
                 framework.stop()
