@@ -6,7 +6,7 @@ Defines the iPOPO decorators classes to manipulate component factory classes
 :author: Thomas Calmant
 :copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.5.7
+:version: 0.5.8
 :status: Beta
 
 ..
@@ -27,7 +27,7 @@ Defines the iPOPO decorators classes to manipulate component factory classes
 """
 
 # Module version
-__version_info__ = (0, 5, 7)
+__version_info__ = (0, 5, 8)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -715,7 +715,7 @@ class Requires(object):
     """ ID of the handler configured by this decorator """
 
     def __init__(self, field, specification, aggregate=False, optional=False,
-                 spec_filter=None):
+                 spec_filter=None, immediate_rebind=False):
         """
         Sets up the requirement
 
@@ -725,6 +725,10 @@ class Requires(object):
         :param optional: If true, this injection is optional
         :param spec_filter: An LDAP query to filter injected services upon
                             their properties
+        :param immediate_rebind: If True, the component won't be invalidated
+                                 then re-validated if a matching service is
+                                 available when the injected dependency is
+                                 unbound
         :raise TypeError: A parameter has an invalid type
         :raise ValueError: An error occurred while parsing the filter or an
                            argument is incorrect
@@ -746,8 +750,9 @@ class Requires(object):
         self.__multi_specs = len(specifications) > 1
 
         # Construct the requirement object
-        self.__requirement = Requirement(specifications[0],
-                                         aggregate, optional, spec_filter)
+        self.__requirement = Requirement(specifications[0], aggregate,
+                                         optional, spec_filter,
+                                         immediate_rebind)
 
     def __call__(self, clazz):
         """
