@@ -8,7 +8,7 @@ Provides the basic command parsing and execution support to make a Pelix shell.
 :author: Thomas Calmant
 :copyright: Copyright 2014, isandlaTech
 :license: Apache License 2.0
-:version: 0.5.7
+:version: 0.5.8
 :status: Beta
 
 ..
@@ -29,7 +29,7 @@ Provides the basic command parsing and execution support to make a Pelix shell.
 """
 
 # Module version
-__version_info__ = (0, 5, 7)
+__version_info__ = (0, 5, 8)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -38,8 +38,9 @@ __docformat__ = "restructuredtext en"
 # ------------------------------------------------------------------------------
 
 # Shell constants
-from pelix.shell import SERVICE_SHELL, SERVICE_SHELL_COMMAND, \
+from . import SERVICE_SHELL, SERVICE_SHELL_COMMAND, \
     SERVICE_SHELL_UTILS
+import pelix.shell.beans as beans
 
 # Pelix modules
 from pelix.utilities import to_str, to_bytes
@@ -47,22 +48,17 @@ import pelix.constants as constants
 import pelix.framework as pelix
 
 # Standard library
+import collections
 import inspect
 import linecache
 import logging
 import os
 import shlex
+import string
 import sys
 import traceback
 import threading
 
-# Before Python 3, input() was raw_input()
-if sys.version_info[0] < 3:
-    safe_input = raw_input
-
-else:
-    safe_input = input
-
 # ------------------------------------------------------------------------------
 
 DEFAULT_NAMESPACE = "default"
@@ -94,13 +90,21 @@ def _find_assignment(arg_token):
     return -1
 
 
-def _make_args(args_list):
+class _ArgTemplate(string.Template):
+    """
+    Argument string template class
+    """
+    idpattern = r'[_a-z\?][_a-z0-9\.]*'
+
+
+def _make_args(args_list, session, fw_props):
     """
     Converts the given list of arguments into a list (args) and a
     dictionary (kwargs).
     All arguments with an assignment are put into kwargs, others in args.
 
     :param args_list: The list of arguments to be treated
+    :param session: The current shell session
     :return: The (arg_token, kwargs) tuple.
     """
     args = []
@@ -113,11 +117,19 @@ def _make_args(args_list):
             key = arg_token[:idx]
             value = arg_token[idx + 1:]
             kwargs[key] = value
-
         else:
             # Direct argument
             args.append(arg_token)
 
+    # Prepare the dictionary of variables
+    variables = collections.defaultdict(str)
+    variables.update(fw_props)
+    variables.update(session.variables)
+
+    # Replace variables
+    args = [_ArgTemplate(arg).safe_substitute(variables) for arg in args]
+    kwargs = dict((key, _ArgTemplate(value).safe_substitute(variables))
+                  for key, value in kwargs.items())
     return args, kwargs
 
 
@@ -149,127 +161,6 @@ def _split_ns_command(cmd_token):
 # ------------------------------------------------------------------------------
 
 
-class IOHandler(object):
-    """
-    Handles I/O operations between the command handler and the client
-    It automatically converts the given data to bytes in Python 3.
-    """
-    def __init__(self, in_stream, out_stream, encoding='UTF-8'):
-        """
-        Sets up the printer
-
-        :param in_stream: Input stream
-        :param out_stream: Output stream
-        :param encoding: Output encoding
-        """
-        self.input = in_stream
-        self.output = out_stream
-        self.encoding = encoding
-
-        # Standard behavior
-        self.flush = self.output.flush
-        self.write = self.output.write
-
-        # Specific behavior
-        if sys.version_info[0] >= 3:
-            if 'b' in getattr(out_stream, 'mode', ''):
-                # Bytes conversion
-                self.write = self._write_bytes
-            else:
-                # Strings accepted
-                self.write = self._write_str
-
-        # Very specific
-        if in_stream is sys.stdin:
-            # Warning: conflicts with the console
-            self.prompt = safe_input
-        else:
-            self.prompt = self._prompt
-
-    def _prompt(self, prompt=None):
-        """
-        Reads a line written by the user
-
-        :param prompt: An optional prompt message
-        :return: The read line, after a conversion to str
-        """
-        if prompt:
-            # Print the prompt
-            self.write(prompt)
-            self.output.flush()
-
-        # Read the line
-        return to_str(self.input.readline())
-
-    def _write_bytes(self, data):
-        """
-        Converts the given data then writes it
-
-        :param data: Data to be written
-        :return: The result of ``self.output.write()``
-        """
-        self.output.write(to_bytes(data, self.encoding))
-
-    def _write_str(self, data):
-        """
-        Converts the given data then writes it
-
-        :param data: Data to be written
-        :return: The result of ``self.output.write()``
-        """
-        self.output.write(to_str(data, self.encoding))
-
-    def write_line(self, line, *args, **kwargs):
-        """
-        Formats and writes a line to the output
-        """
-        if line is None:
-            # Empty line
-            self.write('\n')
-
-        else:
-            # Format the line, if arguments have been given
-            if args or kwargs:
-                line = line.format(*args, **kwargs)
-
-            # Write it
-            self.write(line)
-
-            try:
-                if line[-1] != '\n':
-                    # Add the trailing new line
-                    self.write('\n')
-
-            except IndexError:
-                # Got an empty string
-                self.write('\n')
-
-        self.flush()
-
-    def write_line_no_feed(self, line, *args, **kwargs):
-        """
-        Formats and writes a line to the output
-        """
-        if line is None:
-            # Empty line
-            line = ""
-
-        else:
-            # Format the line, if arguments have been given
-            if args or kwargs:
-                line = line.format(*args, **kwargs)
-
-            # Remove the trailing line feed
-            if line[-1] == '\n':
-                line = line[:-1]
-
-        # Write it
-        self.write(line)
-        self.flush()
-
-# ------------------------------------------------------------------------------
-
-
 class ShellUtils(object):
     """
     Utility methods for the shell
@@ -390,6 +281,7 @@ class Shell(object):
         """
         self._commands = {}
         self._context = context
+        self._framework = context.get_bundle(0)
         self._utils = utilities
 
         # Bound services: reference -> service
@@ -405,6 +297,10 @@ class Shell(object):
         self.register_command(None, "sd", self.service_details)
         self.register_command(None, "sl", self.services_list)
 
+        self.register_command(None, "set", self.var_set)
+        self.register_command(None, "unset", self.var_unset)
+        self.register_command(None, "run", self.run_file)
+
         self.register_command(None, "start", self.start)
         self.register_command(None, "stop", self.stop)
         self.register_command(None, "update", self.update)
@@ -649,10 +545,21 @@ class Shell(object):
         # Command found
         return namespace, command
 
-    def execute(self, cmdline, stdin=sys.stdin, stdout=sys.stdout):
+    def execute(self, cmdline, session=None):
         """
         Executes the command corresponding to the given line
+
+        :param cmdline: Command line to parse
+        :param session: Current shell session
+        :return: True if command succeeded, else False
         """
+        if session is None:
+            # Default session
+            session = beans.ShellSession(
+                beans.IOHandler(sys.stdin, sys.stdout), {})
+
+        assert isinstance(session, beans.ShellSession)
+
         # Split the command line
         if not cmdline:
             return False
@@ -660,14 +567,10 @@ class Shell(object):
         # Convert the line into a string
         cmdline = to_str(cmdline)
 
-        # Prepare the I/O handler
-        io_handler = IOHandler(stdin, stdout)
-
         try:
             line_split = shlex.split(cmdline, True, True)
-
         except ValueError as ex:
-            io_handler.write_line("Error reading line: {0}", ex)
+            session.write_line("Error reading line: {0}", ex)
             return False
 
         if not line_split:
@@ -676,53 +579,53 @@ class Shell(object):
         try:
             # Extract command information
             namespace, command = self.get_ns_command(line_split[0])
-
         except ValueError as ex:
             # Unknown command
-            io_handler.write_line(str(ex))
+            session.write_line(str(ex))
             return False
 
         # Get the content of the name space
         space = self._commands.get(namespace, None)
         if not space:
-            io_handler.write_line("Unknown name space {0}", namespace)
+            session.write_line("Unknown name space {0}", namespace)
             return False
 
         # Get the method object
         method = space.get(command, None)
         if method is None:
-            io_handler.write_line("Unknown command: {0}.{1}", namespace,
-                                  command)
+            session.write_line("Unknown command: {0}.{1}", namespace, command)
             return False
 
         # Make arguments and keyword arguments
-        args, kwargs = _make_args(line_split[1:])
-
-        # Execute it
+        args, kwargs = _make_args(line_split[1:], session,
+                                  self._framework.get_properties())
         try:
-            result = method(io_handler, *args, **kwargs)
-            # None is considered as a success
-            return result is None or result
+            # Execute it
+            result = method(session, *args, **kwargs)
 
+            # Store the result as $?
+            if result is not None:
+                session.set(beans.RESULT_VAR_NAME, result)
+
+            # 0, None are considered as success, so don't use not nor bool
+            return result is not False
         except TypeError as ex:
             # Invalid arguments...
             _logger.error("Error calling %s.%s: %s", namespace, command, ex)
-            io_handler.write_line("Invalid method call: {0}", ex)
-            self.__print_namespace_help(io_handler, namespace, command)
+            session.write_line("Invalid method call: {0}", ex)
+            self.__print_namespace_help(session, namespace, command)
             return False
-
         except Exception as ex:
             # Error
             _logger.exception("Error calling %s.%s: %s",
                               namespace, command, ex)
-            io_handler.write_line("{0}: {1}", type(ex).__name__, str(ex))
+            session.write_line("{0}: {1}", type(ex).__name__, str(ex))
             return False
-
         finally:
             # Try to flush in any case
             try:
-                io_handler.flush()
-            except:
+                session.flush()
+            except IOError:
                 pass
 
     def get_banner(self):
@@ -776,6 +679,60 @@ class Shell(object):
         """
         io_handler.write_line(' '.join(words))
 
+    def var_set(self, session, **kwargs):
+        """
+        Sets the given variables or prints the current ones. "set answer=42"
+        """
+        if not kwargs:
+            session.write_line(
+                self._utils.make_table(('Name', 'Value'),
+                                       session.variables.items()))
+        else:
+            for name, value in kwargs.items():
+                name = name.strip()
+                session.set(name, value)
+                session.write_line("{0}={1}", name, value)
+
+    def var_unset(self, session, name):
+        """
+        Unsets the given variable
+        """
+        name = name.strip()
+        try:
+            session.unset(name)
+        except KeyError:
+            session.write_line("Unknown variable: {0}", name)
+            return False
+        else:
+            session.write_line("Variable {0} unset.", name)
+
+    def run_file(self, session, filename):
+        """
+        Runs the given "script" file
+        """
+        try:
+            with open(filename, "r") as filep:
+                for lineno, line in enumerate(filep):
+                    line = line.strip()
+                    if not line or line.startswith("#"):
+                        # Ignore comments and empty lines
+                        continue
+
+                    # Print out the executed line
+                    session.write_line("[{0:02d}] >> {1}", lineno, line)
+
+                    # Execute the line
+                    if not self.execute(line, session):
+                        session.write_line(
+                            "Command at line {0} failed. Abandon.", lineno + 1)
+                        return False
+
+                session.write_line("Script execution succeeded")
+
+        except IOError as ex:
+            session.write_line("Error reading file {0}: {1}", filename, ex)
+            return False
+
     def bundle_details(self, io_handler, bundle_id):
         """
         Prints the details of the bundle with the given ID or name
@@ -785,7 +742,6 @@ class Shell(object):
         try:
             # Convert the given ID into an integer
             bundle_id = int(bundle_id)
-
         except ValueError:
             # Not an integer, suppose it's a bundle name
             for bundle in self._context.get_bundles():
@@ -794,7 +750,6 @@ class Shell(object):
             else:
                 # Bundle not found
                 bundle = None
-
         else:
             # Integer ID: direct access
             try:
@@ -805,7 +760,7 @@ class Shell(object):
         if bundle is None:
             # No matching bundle
             io_handler.write_line("Unknown bundle ID: {0}", bundle_id)
-            return
+            return False
 
         lines = ["ID......: {0}".format(bundle.get_bundle_id()),
                  "Name....: {0}".format(bundle.get_symbolic_name()),
@@ -821,7 +776,6 @@ class Shell(object):
                     lines.append("\t{0}".format(svc_ref))
             else:
                 lines.append("\tn/a")
-
         except constants.BundleException as ex:
             # Bundle in a invalid state
             lines.append("\tError: {0}".format(ex))
@@ -832,10 +786,8 @@ class Shell(object):
             if services:
                 for svc_ref in services:
                     lines.append("\t{0}".format(svc_ref))
-
             else:
                 lines.append("\tn/a")
-
         except constants.BundleException as ex:
             # Bundle in a invalid state
             lines.append("\tError: {0}".format(ex))
@@ -876,7 +828,6 @@ class Shell(object):
 
         if name is None:
             io_handler.write_line("{0} bundles installed", len(lines))
-
         else:
             io_handler.write_line("{0} filtered bundles", len(lines))
 
@@ -888,7 +839,7 @@ class Shell(object):
             None, '({0}={1})'.format(constants.SERVICE_ID, service_id))
         if svc_ref is None:
             io_handler.write_line('Service not found: {0}', service_id)
-            return
+            return False
 
         lines = [
             "ID............: {0}".format(
@@ -940,6 +891,7 @@ class Shell(object):
         if not lines and specification:
             # No matching service found
             io_handler.write_line("No service provides '{0}'", specification)
+            return False
         else:
             # Print'em all
             io_handler.write(self._utils.make_table(headers, lines))
@@ -987,7 +939,6 @@ class Shell(object):
 
         # Get the documentation string
         doc = inspect.getdoc(method) or "(Documentation missing)"
-
         return ' '.join(args), ' '.join(doc.split())
 
     def __print_command_help(self, io_handler, namespace, cmd_name):
@@ -1148,8 +1099,12 @@ class Shell(object):
         """
         Lists the active threads and their current code line
         """
-        # Extract frames
-        frames = sys._current_frames()
+        try:
+            # Extract frames
+            frames = sys._current_frames()
+        except AttributeError:
+            io_handler.write_line("sys._current_frames() is not available.")
+            return
 
         # Get the thread ID -> Thread mapping
         names = threading._active.copy()
@@ -1173,8 +1128,8 @@ class Shell(object):
             # Construct the code position
             lines.append('Thread ID: {0} - Name: {1}'.format(thread_id, name))
             lines.append('Line:')
-            lines.extend((line.rstrip()
-                          for line in traceback.format_stack(stack, 1)))
+            lines.extend(line.rstrip()
+                         for line in traceback.format_stack(stack, 1))
             lines.append('')
 
         lines.append('')
@@ -1190,18 +1145,16 @@ class Shell(object):
             # Get the stack
             thread_id = int(thread_id)
             stack = sys._current_frames()[thread_id]
-
         except KeyError:
             io_handler.write_line("Unknown thread ID: {0}", thread_id)
-
         except ValueError:
             io_handler.write_line("Invalid thread ID: {0}", thread_id)
-
+        except AttributeError:
+            io_handler.write_line("sys._current_frames() is not available.")
         else:
             # Get the name
             try:
                 name = threading._active[thread_id].name
-
             except KeyError:
                 name = "<unknown>"
 
@@ -1245,8 +1198,8 @@ class Shell(object):
         try:
             # Try to get the type of the calling object
             instance = frame.f_locals['self']
-            method_name = '{0}::{1}'.format(type(instance).__name__,
-                                            method_name)
+            method_name = '{0}::{1}' \
+                .format(type(instance).__name__, method_name)
         except KeyError:
             # Not called from a bound method
             pass
@@ -1348,7 +1301,6 @@ class Shell(object):
                 name,
                 logging.getLevelName(logger.getEffectiveLevel()),
                 logging.getLevelName(logger.level))
-
         else:
             # Set the logger level
             try:
@@ -1377,36 +1329,46 @@ class Shell(object):
         try:
             bundle_id = int(bundle_id)
             return self._context.get_bundle(bundle_id)
-
         except (TypeError, ValueError):
             io_handler.write_line("Invalid bundle ID: {0}", bundle_id)
-
         except constants.BundleException:
             io_handler.write_line("Unknown bundle: {0}", bundle_id)
 
-    def start(self, io_handler, bundle_id):
+    def start(self, io_handler, bundle_id, *bundles_ids):
         """
-        Starts the bundle with the given ID
+        Starts the bundles with the given IDs. Stops on first failure.
         """
-        bundle = self.__get_bundle(io_handler, bundle_id)
-        if bundle is not None:
-            bundle.start()
+        for bid in (bundle_id,) + bundles_ids:
+            bundle = self.__get_bundle(io_handler, bid)
+            if bundle is not None:
+                io_handler.write_line("Starting bundle {0}...", bid)
+                bundle.start()
+            else:
+                return False
 
-    def stop(self, io_handler, bundle_id):
+    def stop(self, io_handler, bundle_id, *bundles_ids):
         """
-        Stops the bundle with the given ID
+        Stops the bundles with the given IDs. Stops on first failure.
         """
-        bundle = self.__get_bundle(io_handler, bundle_id)
-        if bundle is not None:
-            bundle.stop()
+        for bid in (bundle_id,) + bundles_ids:
+            bundle = self.__get_bundle(io_handler, bid)
+            if bundle is not None:
+                io_handler.write_line("Stopping bundle {0}...", bid)
+                bundle.stop()
+            else:
+                return False
 
-    def update(self, io_handler, bundle_id):
+    def update(self, io_handler, bundle_id, *bundles_ids):
         """
-        Updates the bundle with the given ID
+        Updates the bundles with the given IDs. Stops on first failure.
         """
-        bundle = self.__get_bundle(io_handler, bundle_id)
-        if bundle is not None:
-            bundle.update()
+        for bid in (bundle_id,) + bundles_ids:
+            bundle = self.__get_bundle(io_handler, bid)
+            if bundle is not None:
+                io_handler.write_line("Updating bundle {0}...", bid)
+                bundle.update()
+            else:
+                return False
 
     def install(self, io_handler, module_name):
         """
@@ -1414,14 +1376,19 @@ class Shell(object):
         """
         bundle = self._context.install_bundle(module_name)
         io_handler.write_line("Bundle ID: {0}", bundle.get_bundle_id())
+        return bundle.get_bundle_id()
 
-    def uninstall(self, io_handler, bundle_id):
+    def uninstall(self, io_handler, bundle_id, *bundles_ids):
         """
-        Uninstalls the bundle with the given ID
+        Uninstalls the bundles with the given IDs. Stops on first failure.
         """
-        bundle = self.__get_bundle(io_handler, bundle_id)
-        if bundle is not None:
-            bundle.uninstall()
+        for bid in (bundle_id,) + bundles_ids:
+            bundle = self.__get_bundle(io_handler, bid)
+            if bundle is not None:
+                io_handler.write_line("Uninstalling bundle {0}...", bid)
+                bundle.uninstall()
+            else:
+                return False
 
 # ------------------------------------------------------------------------------
 
@@ -1450,7 +1417,6 @@ class PelixActivator(object):
                     pelix.ServiceEvent.MODIFIED):
             # New or modified service
             self._shell._bind_handler(reference)
-
         else:
             # Service gone or not matching anymore
             self._shell._unbind_handler(reference)
@@ -1465,19 +1431,16 @@ class PelixActivator(object):
             # Prepare the shell utility service
             utils = ShellUtils()
             self._shell = Shell(context, utils)
-
-            self._shell_reg = context.register_service(SERVICE_SHELL,
-                                                       self._shell, {})
-
-            self._utils_reg = context.register_service(SERVICE_SHELL_UTILS,
-                                                       utils, {})
+            self._shell_reg = context.register_service(
+                SERVICE_SHELL, self._shell, {})
+            self._utils_reg = context.register_service(
+                SERVICE_SHELL_UTILS, utils, {})
 
             # Register the service listener
             context.add_service_listener(self, None, SERVICE_SHELL_COMMAND)
 
             # Register existing command services
-            refs = context.get_all_service_references(SERVICE_SHELL_COMMAND,
-                                                      None)
+            refs = context.get_all_service_references(SERVICE_SHELL_COMMAND)
             if refs is not None:
                 for ref in refs:
                     self._shell._bind_handler(ref)
